/****
 * CHANGELOG
 * [12/4/2008 maw] Made it possible to also send attachments with MIME
 *                 type text/*
 ****/
package org.ten60.smtp.accessor;

import com.ten60.netkernel.urii.IURRepresentation;
import com.ten60.netkernel.urii.aspect.IAspectBinaryStream;
import org.ten60.netkernel.layer1.util.CompoundURIdentifier;
import org.ten60.netkernel.xml.representation.IAspectXDA;
import org.ten60.netkernel.xml.xahelper.XAHelper;
import org.ten60.netkernel.xml.xahelper.XAHelperExtra;
import org.ten60.netkernel.xml.xahelper.XAccessor;
import org.ten60.netkernel.xml.xda.IXDAReadOnly;
import org.ten60.netkernel.xml.xda.IXDAReadOnlyIterator;

import javax.activation.DataHandler;
import javax.mail.Message;
import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.net.URI;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Common functionality shared between both the queueing accessor (SMTPAccessor) and
 * the sync accessor (SMTPSyncAccessor).  Both extend this class.
 */
public abstract class BaseSMTPAccessor extends XAccessor {
    final static String MESSAGE_ARGUMENT = "part";

    protected Session mSession;

    protected String mMailAddress;
    protected String mSMTPGateway;
    protected String mSMTPUser;
    protected String mSMTPPassword;
	protected int mSMTPPort=-1;

    protected boolean mConfigured = false;

    public BaseSMTPAccessor() {
        declareDisableArgumentChecking();
    }

    /**
     * Configure common Email settings
     *
     * @param aHelper The helper class that provides access to the config,
     */
    protected void configure( XAHelper aHelper ) throws Exception {
        preConfigure();
        URI configURI = URI.create( "ffcpl:/etc/SMTPConfig.xml" );
        IURRepresentation rep = aHelper.getResource( configURI, IAspectXDA.class );
        IAspectXDA config = ( IAspectXDA ) rep.getAspect( IAspectXDA.class );
        IXDAReadOnly aConfig = config.getXDA();

        mSMTPGateway = aConfig.getText( "/SMTPConfig/SMTPGateway", true );
        mSMTPUser = aConfig.getText( "/SMTPConfig/SMTPUser", true );
        mSMTPPassword = aConfig.getText( "/SMTPConfig/SMTPPassword", true );
        mMailAddress = aConfig.getText( "/SMTPConfig/sender", true );
		if(aConfig.isTrue("/SMTPConfig/SMTPPort"))
		{	mSMTPPort=Integer.parseInt(aConfig.getText("/SMTPConfig/SMTPPort", true));			
		}

        //Create mail session
        StringBuffer mailsettings = new StringBuffer( "mail.store.protocol=pop3\n" );
        mailsettings.append( "mail.transport.protocol=smtp\n" );

        ByteArrayInputStream bis = new ByteArrayInputStream( mailsettings.toString().getBytes() );
        java.util.Properties ps = new java.util.Properties();
        ps.load( bis );
        mSession = Session.getInstance( ps );
        mConfigured = true;
        postConfigure();
    }

    /**
     * Abstract method to be implemented in subclass.
     *
     * @param aHelper The helper class that provides access to the operator,
     *                operand, and parameters.
     * @return Either the result as a Document or a Reader.
     */
    public IURRepresentation source( XAHelper aHelper ) throws Exception {
        Thread.currentThread().setContextClassLoader( this.getClass().getClassLoader() );
        if( !mConfigured || ( aHelper.hasOperator() && aHelper.getOperator().getXDA().isTrue( "/reconfigure" ) ) ) {
            configure( aHelper );
        }
        if( !aHelper.hasOperator() ) {
            IXDAReadOnly opd = aHelper.getOperand().getXDA();

            MimeMessage message = new MimeMessage( mSession );
            message.setSubject( opd.getText( "/email/subject", true ) );
            String from = null;
            if( opd.isTrue( "/email/from" ) ) {
                from = opd.getText( "/email/from", true );
            } else from = mMailAddress;
            InternetAddress IAfrom = new InternetAddress( from );
            message.setSender( IAfrom );
            message.setFrom( IAfrom );

            IXDAReadOnlyIterator ir = opd.readOnlyIterator( "/email/to" );
            Map map = new HashMap( 5 );
            while( ir.hasNext() ) {
                ir.next();
                String to = ir.getText( ".", true );
                if( !map.containsKey( to ) ) {
                    message.addRecipient( Message.RecipientType.TO, new InternetAddress( to ) );
                    map.put( to, Boolean.TRUE );
                }
            }

            ir = opd.readOnlyIterator( "/email/cc" );
            map = new HashMap( 5 );
            while( ir.hasNext() ) {
                ir.next();
                String cc = ir.getText( ".", true );
                if( !map.containsKey( cc ) ) {
                    message.addRecipient( Message.RecipientType.CC, new InternetAddress( cc ) );
                    map.put( cc, Boolean.TRUE );
                }
            }
            ir = opd.readOnlyIterator( "/email/bcc" );
            map = new HashMap( 5 );
            while( ir.hasNext() ) {
                ir.next();
                String bcc = ir.getText( ".", true );
                if( !map.containsKey( bcc ) ) {
                    message.addRecipient( Message.RecipientType.BCC, new InternetAddress( bcc ) );
                    map.put( bcc, Boolean.TRUE );
                }
            }

            //Add Messages
            MimeMultipart replymp = new MimeMultipart();
            XAHelperExtra extra = ( XAHelperExtra ) aHelper;
            CompoundURIdentifier curi = new CompoundURIdentifier( extra.getRequest().getURI() );
            Iterator i = curi.getArgs();
            boolean messageflag = false;
            while( i.hasNext() ) {
                CompoundURIdentifier.CompoundURIStruct struct = ( CompoundURIdentifier.CompoundURIStruct ) i.next();
                if( !struct.getKey().equals( "operand" ) ) {
                    messageflag = true;
                    IURRepresentation rep = aHelper.getResource( URI.create( struct.getURI() ), IAspectBinaryStream.class );
                    IAspectBinaryStream bsAspect = ( IAspectBinaryStream ) rep.getAspect( IAspectBinaryStream.class );
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    bsAspect.write( baos );
                    baos.flush();
                    MimeBodyPart mbp = new MimeBodyPart();
                    DataHandler dh = null;
                    if( !rep.getMeta().getMimeType().startsWith( "text" ) || aHelper.getOperand().getXDA().isTrue( "//attachment[name='" + struct.getKey() + "']") ) {
                        System.err.println("DBG: "+struct.getKey()+":handled as attachment");
                        ByteArrayDataSource bads = new ByteArrayDataSource( baos.toByteArray(), rep.getMeta().getMimeType() );
                        dh = new DataHandler( bads );
                        try {
                            mbp.setFileName( aHelper.getOperand().getXDA().getText( "//attachment[name='" + struct.getKey() + "']/filename", true ) );
                        }
                        catch( Exception e ) {
                            mbp.setFileName( struct.getKey() );
                        }
                    } else {
                        System.err.println("DBG: "+struct.getKey()+":handled as part");
                        dh = new DataHandler( baos.toString(), rep.getMeta().getMimeType() );
                    }
                    mbp.setDataHandler( dh );
                    replymp.addBodyPart( mbp );
                }
            }
            if( !messageflag ) throw new Exception( "Must provide at least one message part argument" );
            message.setContent( replymp );
            message.saveChanges();

            messageAction( message );
        }
        return org.ten60.netkernel.layer1.representation.VoidAspect.create();
    }

    protected abstract void preConfigure() throws Exception;

    protected abstract void postConfigure() throws Exception;

    protected abstract void messageAction( MimeMessage message ) throws Exception;

}
